/*
#
# Copyright (C) 2011, audin.
# GlcdSt7565-v02: 2011/10 : Modified for STM32VLDiscovery.  
# GlcdSt7565-v01: 2011/09 : Modified for ST7565 SPI.
# This soft is under LGPL v2.1.
# http://avr.paslog.jp
#
*/

#ifndef AD12864_SPI_MANUAL_CONFIG_H
#define AD12864_SPI_MANUAL_CONFIG_H

#define GLCD_NOINIT_CHECKS
#define GLCD_READ_CACHE       // Turns on code that uses a frame buffer for a read cache

/*********************************************************/
/*  Configuration for LCD panel specific configuration   */
/*********************************************************/
#define DISPLAY_WIDTH 128
#define DISPLAY_HEIGHT 64

// panel controller chips
#define CHIP_WIDTH     128  // pixels per chip 
#define CHIP_HEIGHT    64  // pixels per chip

/*********************************************************/
/*  Configuration for assigning LCD bits to Arduino Pins */
/*********************************************************/

/*
 * define name for pin configuration
 */
#define glcd_ConfigName "st7565-Manual"

// calculate number of chips & round up if width is not evenly divisable
#define glcd_CHIP_COUNT 1 // (((DISPLAY_WIDTH + CHIP_WIDTH - 1)  / CHIP_WIDTH) * ((DISPLAY_HEIGHT + CHIP_HEIGHT -1) / CHIP_HEIGHT))


#define SPI_NUMBER 1			/* Select SPI Number */
#if defined( BOARD_STM32VLD )
/* GLCD-Maple Pin Assign for STM32VLDiscovery
GLCD  pin:  LED_A | GND | 3.3V | SDI | SCK | A0 | RST || CS1
Maple pin:                       PA7   PA5   PC4  PC5    PB0
*/
#    define glcdRES            PC(5)   // Reset Bit RST
#    define glcdCS1            PB(0)   // chip sel CS1
#    define glcdDI             PC(4)   // A0/DI
#    define SPI_CLOCK_VALUE  SPI_3MHZ 
#else
/* GLCD-Maple Pin Assign for other Maple family.
GLCD  pin:  LED_A | GND | 3.3V | SDI | SCK | A0 | RST || CS1
Maple pin:                       D11   D13   D10   D9     D8
*/
#    define glcdRES            9   // Reset Bit RST
#    define glcdCS1            8   // chip sel CS1
#    define glcdDI            10   // A0/DI
#    define SPI_CLOCK_VALUE  SPI_2_25MHZ  // SPI_562_500KHZ, SPI_2_25MHZ, SPI_4_5MHZ,SPI_9MHZ, SPI_18MHZ
#endif

/* For source compatibilty */ 
#define DO_NOP   255
#define glcdRW   DO_NOP   /* Indicate do noting */

//#define glcdCSEL1         
//#define glcdCSEL2 
//#define glcdEN            

#include "device/st7565_Device.h"

#endif // AD12864_SPI_MANUAL_CONFIG_H
