/*
  glcd_io.h
  Copyright (c) 2009 Bill Perry
  
  vi:ts=4

  This file is part of the Arduino GLCD library.

  GLCD is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.

  GLCD is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with GLCD.  If not, see <http://www.gnu.org/licenses/>.

  This file maps abstract io requests from glcd_Device to AVR port and pin abstractions
  arduino_avrio.h maps arduino pins to avr ports and pins.
  The physical io is handled by macros in avrio.h
 
 */

#ifndef	GLCD_IO_H
#define GLCD_IO_H

#include "wirish.h"

// lcdfastWrite Macro may be replaced by Paul's new Arduino macro 
#define lcdfastWrite(pin, pinval){ if( pin != DO_NOP) digitalWrite(pin, pinval); }
#define lcdPinMode(pin, mode)    { if( pin != DO_NOP) pinMode(pin, mode); }

#if defined( ST7565_AD_12864_SPI )
/*
 * alias to read status bits
 */
#define lcdRdBusystatus()	0	
#define lcdRdResetstatus()	
/*
 * alias to check status bits
 */
#define lcdIsBusyStatus(status)		0 
#define lcdIsResetStatus(status)	0

#    define mSpi_Lcd_CS_enable()        digitalWrite(glcdCS1, LOW)    //  active low
#    define mSpi_Lcd_CS_disable()       digitalWrite(glcdCS1, HIGH)   //
#    define mSpi_LCD_send_byte(c)       _SPI_OBJ.write(c)
#    define mSpi_LCD_read_byte()        _SPI_OBJ.read()
#    define mSpi_LCD_transfer_byte(c)   _SPI_OBJ.transfer(c)
#    define lcd_data_write(d)	        { lcdfastWrite(glcdDI, HIGH); lcdDataOut( d); }
#    define lcdDataOut(data)	        { mSpi_Lcd_CS_enable(); \
                                          mSpi_LCD_transfer_byte( data ); \
                                          mSpi_Lcd_CS_disable();}
#    define lcdDataDir(dirbits)	        /* No operation   for dummy */
#    define delay_nano(n)               /* No operation   for dummy */
#    define lcdIsBusy()                 0 /* Always false for dummy */ 
#    define lcdDataIn()                 0 /* Always 0     for dummy */
											
#else
#define nano140() { nano70(); nano70(); }
#define nano70() { \
                    asm volatile("nop"); \
                    asm volatile("nop"); \
                    asm volatile("nop"); \
                    asm volatile("nop"); \
	                asm volatile("nop"); }


void inline delay_nano( int n )
{
	n = n / 140;
	while(n--){
		nano140();
	}
	nano70();
}

/*
 * Configure the direction of the data pins.
 *	0x00 is for input and 0xff is for output.
 */
#    define lcdDataDir(dirbits)				\
	    set_port_dir(dirbits, 			    \
			glcdData0Pin, glcdData1Pin,		\
			glcdData2Pin, glcdData3Pin,		\
			glcdData4Pin, glcdData5Pin,		\
			glcdData6Pin, glcdData7Pin)

#    define lcdDataOut(data) \
	     set_port_data(data, 			    \
			glcdData0Pin, glcdData1Pin,		\
			glcdData2Pin, glcdData3Pin,		\
			glcdData4Pin, glcdData5Pin,		\
			glcdData6Pin, glcdData7Pin)

#    define lcdDataIn() \
	     read_port_data( 			        \
			glcdData0Pin, glcdData1Pin,		\
			glcdData2Pin, glcdData3Pin,		\
			glcdData4Pin, glcdData5Pin,		\
			glcdData6Pin, glcdData7Pin)

#define xGLCD_STATUS_BIT2PIN(bit)    glcdData ## bit  ## Pin
#define GLCD_STATUS_BIT2PIN(bit)     xGLCD_STATUS_BIT2PIN(bit)    
#define lcdIsBusy()		            ( digitalRead( GLCD_STATUS_BIT2PIN( LCD_BUSY_BIT  ) ) )
//#define lcdIsReset()		        ( digitalRead( GLCD_STATUS_BIT2PIN( LCD_RESET_BIT ) ) )

#define INLINE inline
static INLINE void set_port_dir(int dir, int p0, int p1, int p2, int p3,int p4, int p5,int p6, int p7 )
{
	WiringPinMode mode;
	mode = ( dir ? OUTPUT: INPUT );
	pinMode( p0, mode );
	pinMode( p1, mode );
	pinMode( p2, mode );
	pinMode( p3, mode );
	pinMode( p4, mode );
	pinMode( p5, mode );
	pinMode( p6, mode );
	pinMode( p7, mode );	
}

static INLINE void set_port_data(int data, int p0, int p1, int p2, int p3,int p4, int p5,int p6, int p7 )
{
#if 0
	(data & 0x01) ? digitalWrite(p0, HIGH) : digitalWrite(p0, LOW);
	(data & 0x02) ? digitalWrite(p1, HIGH) : digitalWrite(p1, LOW);
	(data & 0x04) ? digitalWrite(p2, HIGH) : digitalWrite(p2, LOW);
	(data & 0x08) ? digitalWrite(p3, HIGH) : digitalWrite(p3, LOW);
	(data & 0x10) ? digitalWrite(p4, HIGH) : digitalWrite(p4, LOW);
	(data & 0x20) ? digitalWrite(p5, HIGH) : digitalWrite(p5, LOW);
	(data & 0x40) ? digitalWrite(p6, HIGH) : digitalWrite(p6, LOW);
	(data & 0x80) ? digitalWrite(p7, HIGH) : digitalWrite(p7, LOW);
#endif
	digitalWrite(p0, LOW);
	digitalWrite(p1, LOW);
	digitalWrite(p2, LOW);
	digitalWrite(p3, LOW);
	digitalWrite(p4, LOW);
	digitalWrite(p5, LOW);
	digitalWrite(p6, LOW);
	digitalWrite(p7, LOW);

	if(data & 0x01) digitalWrite(p0, HIGH) ;
	if(data & 0x02) digitalWrite(p1, HIGH) ;
	if(data & 0x04) digitalWrite(p2, HIGH) ;
	if(data & 0x08) digitalWrite(p3, HIGH) ;
	if(data & 0x10) digitalWrite(p4, HIGH) ;
	if(data & 0x20) digitalWrite(p5, HIGH) ;
	if(data & 0x40) digitalWrite(p6, HIGH) ;
	if(data & 0x80) digitalWrite(p7, HIGH) ;
}

static INLINE int read_port_data(int p0, int p1, int p2, int p3,int p4, int p5,int p6, int p7 )
{
	int r ;
	r = digitalRead( p7 );
	r = digitalRead( p6 ) + ( r << 1 );
	r = digitalRead( p5 ) + ( r << 1 );
	r = digitalRead( p4 ) + ( r << 1 );
	r = digitalRead( p3 ) + ( r << 1 );
	r = digitalRead( p2 ) + ( r << 1 );
	r = digitalRead( p1 ) + ( r << 1 );
	r = digitalRead( p0 ) + ( r << 1 );
    return r;
}

#endif /* not ST7565_AD_12864_SPI */


#ifdef glcdRES
#define lcdReset()		digitalWrite(glcdRES, 0)
#define lcdUnReset()	digitalWrite(glcdRES, 1)
#else
#define lcdReset()		
#define lcdUnReset()		
#endif



/*
 * Delay functions
 */

/*
 * For nano second delays there are two real options:
 * The AVR libC supplied <util/delay.h> is not good eough,
 * there is too much error in their calculations.
 *
 * - Hans Heinrichs _delay_cycles()
 * - avr-gcc __builtin_avr_delay_cycles() routine.
 *
 * The two generate very similar code, however the gcc
 * builtin function is only avialable in newer gcc revisions
 * (some older linux avr-gcc revisions don't have it) and Hans' code
 * is sometimes smaller and doesn't use loops which require a
 * a register when the number cycles is less than 12.
 */
#if defined( __AVR__ )
#include "include/delay.h" // Hans' Heirichs delay macros

#define lcdDelayNanoseconds(__ns) _delay_cycles( (double)(F_CPU)*((double)__ns)/1.0e9 + 0.5 ) // Hans Heinrichs delay cycle routine
#endif

#define lcdDelayMilliseconds(__ms) delay(__ms)	// Arduino delay function
#define lcdDelayNanoseconds(n)     delay_nano(n)

/*
 * functions to perform chip selects on panel configurations
 * that use them.
 */

#ifdef glcd_CHIP0 // check to see if panel uses chip selects


/*
 * First some sanity checks
 */

#ifndef glcd_CHIP_COUNT
#error glcd_CHIP_COUNT not defined
#endif

/*
 * The actual functions/macros that do the chip selects.
 * each macro is passed a chipselect string defined
 * in the config file. This string has 1, 2, 3, 4, ....
 * pairs of data that define pins that need to be set to
 * a high/low value.
 * These macros rip those strings apart and generate the
 * functions to set the pin values.
 */

#if (glcd_CHIP_COUNT > 4)
#error GLCD chip count beyond max of 4
#endif

#if defined(glcdCSEL4)
#define lcdChipSelect(cselstr) lcdChipSelect4(cselstr)
#elif defined(glcdCSEL3)
#define lcdChipSelect(cselstr) lcdChipSelect3(cselstr)
#elif defined(glcdCSEL2)
#define lcdChipSelect(cselstr) lcdChipSelect2(cselstr)
#elif defined(glcdCSEL1)
#define lcdChipSelect(cselstr) lcdChipSelect1(cselstr)
#endif

#define lcdChipSelect1(p,v) lcdfastWrite(p,v)

#define lcdChipSelect2(p1,v1, p2,v2) \
 do {lcdChipSelect1(p1, v1); lcdChipSelect1(p2,v2);} while(0)

#define lcdChipSelect3(p1,v1, p2,v2, p3,v3) \
 do {lcdChipSelect1(p1, v1); lcdChipSelect1(p2,v2); lcdChipSelect1(p3,v3);} while(0)

#define lcdChipSelect4(p1,v1, p2,v2, p3,v3, p4,v4) \
 do {lcdChipSelect1(p1, v1); lcdChipSelect1(p2,v2); lcdChipSelect1(p3,v3); lcdChipSelect1(p4,v4);} while(0)

#endif // glcd_CHIP0

#endif // GLCD_IO_H
